//! The data models used by Yorokobot and that link to the MongoDB collections.

use serde::{Deserialize, Serialize};

/// Name of the collections of the database.
pub const COLLECTIONS_NAMES: [&str; 1] = ["guilds"];

/// The trait that should be implemented by the collections model of Yorokobot.
pub trait YorokobotCollection: for<'de> Deserialize<'de> + Serialize + Unpin + Send + Sync {
    /// Return the collection name linked to the model struct.
    fn get_collection_name() -> String;
}

/// Represent a Yorokobot tag.
#[derive(Debug, Serialize, Deserialize, Clone)]
pub struct Tag {
    pub name: String,
    pub is_nsfw: bool,
    pub subscribers: Vec<String>,
}

/// Represent a Discord server/guild.
#[derive(Debug, Serialize, Deserialize, Clone)]
pub struct Guild {
    #[serde(rename = "_id")]
    pub id: String,
    pub tags: Vec<Tag>,
}

impl YorokobotCollection for Guild {
    fn get_collection_name() -> String {
        COLLECTIONS_NAMES[0].to_string()
    }
}
